-- Vector calculations

-- globals
a = {x = 0, y = 0, z = 0}		-- first vector
b = {x = 0, y = 0, z = 0}		-- second vector

-- calculate magnitude(vector)
function mag3(v)
	local mag = sqrt(v.x * v.x + v.y * v.y + v.z * v.z)
	return mag
end

-- calculate v1 dot v2
function dot3(v1,v2)
	local dot = v1.x * v2.x + v1.y * v2.y + v1.z * v2.z
	return dot
end



-- get vector a from the stack and save in global
function load_A(x,y,z)
	local v = {x = x, y = y, z = z}
	a = v
	return
end

-- get vector b from the stack and save in global
function load_B(x,y,z)
	local v = {x = x, y = y, z = z}
	b = v
	return
end

-- calculate magnitude(a.x,a.y,a.z) and put on the stack
function put_mag_A()
	return mag3(a)
end

-- calculate magnitude(b.x,b.y,b.z) and put on the stack
function put_mag_B()
	return mag3(b)
end

-- calculate A•B on the stack
function AdotB()
	return dot3(a,b)
end

-- calculate angle between A,B on the stack (degree)
function AangleB()
	local dot = dot3(a,b)
	local A = mag3(a)
	local B = mag3(b)
	local angle = NaN

	if (A>0 and B>0) then
		angle = acos(dot/(A*B)) * 180 / pi
	end
	return angle	
end

-- computes a/|A|, unit vector, put on the stack
function Ahat()
	local A = mag3(a)
	local nx=0, ny=0, nz=0
	if (A>0) then
		nx = a.x/A
		ny = a.y/A
		nz = a.z/A
	end
	return nx,ny,nz
end

-- computes b/|B|, unit vector, put on the stack
function Bhat()
	local B = mag3(b)
	local nx=0, ny=0, nz=0
	if (B>0) then
		nx = b.x/B
		ny = b.y/B
		nz = b.z/B
	end
	return nx,ny,nz
end

-- put AxB on the stack
function AcrossB()
	local cx = a.y * b.z - a.z * b.y
	local cy = a.z * b.x - a.x * b.z
	local cz = a.x * b.y - a.y * b.x
	return cx,cy,cz
end

--	Button layout
--	A>		B>		|A|		|B|
--	A•B		AB°		A^		B^
--	AxB
--
--	button "A>"		load vector A from x,y,z to global variables a.x, a.y, a.z
--	button "B>"		load vector B from x,y,z to global variables b.x, b.y, b.z
--	button "|A|"	returns |A|
--	button "|B|"	returns |B|
--
--	button "A•B"	return A dot B,  the dot product
--	button	"AB°"	returns angle between A and B (degree)
--	button "A^"		return A/|A|, writes x,y,z
--	button "B^"		return B/|B|, writes x,y,z
--
--	button "AxB"	cross product, writes x,y,z

